-- CopyRight(c) pgRouting developers
-- Creative Commons Attribution-Share Alike 3.0 License : https://creativecommons.org/licenses/by-sa/3.0/
/* -- q1 */
SELECT ST_Area(pgr_alphaShape((SELECT ST_Collect(geom)
      FROM vertices), 1.5));
/* -- q2 */
